<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MWishlist\Model\Repository\WishlistItemRepository;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\Resolver\ArgumentsProcessorInterface;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;

class AddToWishlistFromCart implements ResolverInterface
{
    /**
     * @var WishlistItemRepository
     */
    private $wishlistItemRepository;

    /**
     * @var ArgumentsProcessorInterface
     */
    private $argsSelection;

    public function __construct(
        WishlistItemRepository $wishlistItemRepository,
        ArgumentsProcessorInterface $argsSelection
    ) {
        $this->wishlistItemRepository = $wishlistItemRepository;
        $this->argsSelection = $argsSelection;
    }

    /**
     * @param Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return array
     * @throws GraphQlInputException
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $processedArgs = $this->argsSelection->process($info->fieldName, $args);
        $input = $processedArgs['input'];
        $customerId = $context->getUserId();

        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        try {
            return ['status' => $this->wishlistItemRepository->addProductFromCartToWishlist(
                $processedArgs['wishlistId'],
                $input['cart_item_id'],
                $customerId
            )];
        } catch (LocalizedException $e) {
            throw new GraphQlInputException(__($e->getMessage()));
        }
    }
}
