<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MWishlist\Model\Repository\WishlistRepository;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\Resolver\ArgumentsProcessorInterface;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;

class CreateNewWishlist implements ResolverInterface
{
    /**
     * @var WishlistRepository
     */
    private $wishlistRepository;

    /**
     * @var ArgumentsProcessorInterface
     */
    private $argsSelection;

    public function __construct(
        WishlistRepository $wishlistRepository,
        ArgumentsProcessorInterface $argsSelection
    ) {
        $this->wishlistRepository = $wishlistRepository;
        $this->argsSelection = $argsSelection;
    }

    /**
     * @param Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return array
     * @throws GraphQlInputException
     * @throws CouldNotSaveException
     * @throws GraphQlAuthorizationException
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $processedArgs = $this->argsSelection->process($info->fieldName, $args);
        $customerId = $context->getUserId();

        /* Guest checking */
        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        try {
            $wishlist = $this->wishlistRepository->create();
            if ($processedArgs['input']) {
                $wishlist->setData($processedArgs['input']);
                $wishlist->setData('customer_id', $customerId);
            }
            $wishlist->generateSharingCode();
            $this->wishlistRepository->save($wishlist);
        } catch (CouldNotSaveException $exception) {
            throw new GraphQlInputException(__('Unable to save new wishlist. Error: %1', $exception->getMessage()));
        }

        return ['amWishlist' => $wishlist->getData()];
    }
}
