<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MultipleWishlistGraphqlByAmasty\Model\Wishlist\Filter\SearchCriteria;
use Amasty\MWishlist\Api\Data\WishlistInterface;
use Amasty\MWishlist\Model\ConfigProvider;
use Amasty\MWishlist\Model\Repository\WishlistRepository;
use Amasty\MWishlist\Model\ResourceModel\Wishlist\Collection as WishlistCollection;
use Amasty\MWishlist\Model\Source\Type;
use Amasty\MWishlist\Model\Wishlist as WishlistModel;
use Amasty\MWishlist\Model\Wishlist\Management as WishlistManagement;
use Amasty\MWishlist\ViewModel\PostHelper;
use Magento\Customer\CustomerData\SectionSourceInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Magento\Framework\UrlInterface;
use Magento\Framework\GraphQl\Query\Resolver\ArgumentsProcessorInterface;

class GetRecentlyUpdatedLists implements ResolverInterface
{
    /**
     * @var WishlistManagement
     */
    private $wishlistManagement;

    /**
     * @var UrlInterface
     */
    private $urlBuilder;

    /**
     * @var ArgumentsProcessorInterface
     */
    private $argsSelection;

    /**
     * @var SearchCriteria
     */
    private $searchCriteria;

    /**
     * @var WishlistRepository
     */
    private $wishlistRepository;

    public function __construct(
        WishlistManagement $wishlistManagement,
        UrlInterface $urlBuilder,
        ArgumentsProcessorInterface $argsSelection,
        SearchCriteria $searchCriteria,
        WishlistRepository $wishlistRepository
    ) {
        $this->wishlistManagement = $wishlistManagement;
        $this->urlBuilder = $urlBuilder;
        $this->argsSelection = $argsSelection;
        $this->searchCriteria = $searchCriteria;
        $this->wishlistRepository = $wishlistRepository;
    }

    /**
     * @param Field $field
     * @param $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return array
     * @throws GraphQlAuthorizationException
     * @throws GraphQlInputException
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $processedArgs = $this->argsSelection->process($info->fieldName, $args);
        $customerId = $context->getUserId();

        /* Guest checking */
        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        $searchCriteria = $this->searchCriteria->buildCriteriaForRecently($processedArgs, $customerId);

        try {
            /** @var WishlistInterface $data */
            $wishlists = $this->wishlistRepository->getList($searchCriteria)->getItems();
            foreach ($wishlists as $wishlist) {
                $wishlist['items'] = $wishlist->getWishlistItems();
            }
        } catch (NoSuchEntityException $e) {
            throw new GraphQlInputException(__($e->getMessage()));
        }

        return $wishlists;
    }
}
