<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MWishlist\Api\Data\WishlistInterface;
use Amasty\MWishlist\Model\Repository\WishlistRepository;
use Amasty\MultipleWishlistGraphqlByAmasty\Model\Wishlist\Filter\SearchCriteria;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\Resolver\ArgumentsProcessorInterface;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;

class Wishlists implements ResolverInterface
{
    /**
     * @var WishlistRepository
     */
    private $wishlistRepository;

    /**
     * @var ArgumentsProcessorInterface
     */
    private $argsSelection;

    /**
     * @var SearchCriteria
     */
    private $searchCriteria;

    public function __construct(
        WishlistRepository $wishlistRepository,
        ArgumentsProcessorInterface $argsSelection,
        SearchCriteria $searchCriteria
    ) {
        $this->wishlistRepository = $wishlistRepository;
        $this->argsSelection = $argsSelection;
        $this->searchCriteria = $searchCriteria;
    }

    /**
     * @param Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return array
     * @throws NoSuchEntityException
     * @throws GraphQlInputException
     * @throws GraphQlAuthorizationException
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $processedArgs = $this->argsSelection->process($info->fieldName, $args);
        $customerId = $context->getUserId();

        /* Guest checking */
        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        $searchCriteria = $this->searchCriteria->buildCriteria($processedArgs, $customerId);

        try {
            /** @var WishlistInterface $data */
            $wishlists = $this->wishlistRepository->getList($searchCriteria)->getItems();
            foreach ($wishlists as $wishlist) {
                $wishlist['items'] = $wishlist->getWishlistItems();
            }
        } catch (NoSuchEntityException $e) {
            throw new GraphQlInputException(__($e->getMessage()));
        }

        return $wishlists;
    }
}
