<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Wishlist\Filter;

use Magento\Framework\Api\SearchCriteria as MagentoSearchCriteria;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrderBuilder;

class SearchCriteria
{
    /**
     * @var WishlistFilter
     */
    private $wishlistFilter;

    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @var SortOrderBuilder
     */
    private $sortOrderBuilder;

    public function __construct(
        WishlistFilter $wishlistFilter,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SortOrderBuilder $sortOrderBuilder
    ) {
        $this->wishlistFilter = $wishlistFilter;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;
    }

    /**
     * Transform raw criteria data into SearchCriteriaInterface
     */
    public function buildCriteria(array $criteria, int $customerId): MagentoSearchCriteria
    {
        $filterGroups = $this->wishlistFilter->createFilterGroups($criteria);
        $this->searchCriteriaBuilder->setFilterGroups($filterGroups);
        $pageSize = $criteria['pageSize'] ?? 20;
        $currentPage = $criteria['currentPage'] ?? 1;
        $this->searchCriteriaBuilder
            ->setPageSize($pageSize)
            ->setCurrentPage($currentPage)
            ->addFilter('customer_id', $customerId);

        return $this->searchCriteriaBuilder->create();
    }

    public function buildCriteriaForRecently(array $criteria, int $customerId): MagentoSearchCriteria
    {
        $filterGroups = $this->wishlistFilter->createFilterGroups($criteria);
        $this->searchCriteriaBuilder->setFilterGroups($filterGroups);
        $pageSize = $criteria['pageSize'] ?? 20;
        $currentPage = $criteria['currentPage'] ?? 1;
        $sortOrder = $this->sortOrderBuilder->setField('updated_at')->setDescendingDirection()->create();
        $this->searchCriteriaBuilder
            ->setPageSize($pageSize)
            ->setCurrentPage($currentPage)
            ->addSortOrder($sortOrder)
            ->addFilter('customer_id', $customerId);

        return $this->searchCriteriaBuilder->create();
    }
}
