<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Wishlist\Filter;

use Magento\Framework\Api\Filter;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\Search\FilterGroupBuilder;

class WishlistFilter
{
    /**
     * Translator field from graphql to collection field
     * @var string[]
     */
    private $fieldTranslatorArray = [
        'id' => 'wishlist_id',
    ];

    /**
     * @var string
     */
    private const FILTER = 'filter';

    /**
     * @var FilterBuilder
     */
    private $filterBuilder;

    /**
     * @var FilterGroupBuilder
     */
    private $filterGroupBuilder;

    public function __construct(
        FilterBuilder $filterBuilder,
        FilterGroupBuilder $filterGroupBuilder,
        array $fieldTranslatorArray = []
    ) {
        $this->filterBuilder = $filterBuilder;
        $this->filterGroupBuilder = $filterGroupBuilder;
        $this->fieldTranslatorArray = array_merge($this->fieldTranslatorArray, $fieldTranslatorArray);
    }

    public function createFilterGroups(array $args): array
    {
        $filterGroups = [];

        if (isset($args[self::FILTER])) {
            $filters = [];
            foreach ($args[self::FILTER] as $field => $cond) {
                if (isset($this->fieldTranslatorArray[$field])) {
                    $field = $this->fieldTranslatorArray[$field];
                }

                foreach ($cond as $condType => $value) {
                    $filters[] = $this->createFilter($field, $value, $condType);
                }
            }

            $this->filterGroupBuilder->setFilters($filters);
            $filterGroups[] = $this->filterGroupBuilder->create();
        }

        return $filterGroups;
    }

    /**
     * @param string $field
     * @param string|int|array $value
     * @param string $conditionType
     * @return Filter
     */
    private function createFilter(string $field, $value, string $conditionType = 'eq'): Filter
    {
        $this->filterBuilder->setField($field);
        $this->filterBuilder->setValue($value);
        $this->filterBuilder->setConditionType($conditionType);

        return $this->filterBuilder->create();
    }
}
